// Parser za mikroC (samo gramatika)
%{
    #include <stdio.h>
    #include "defs.h"

    int yyparse(void);
    int yylex(void);
    int yyerror(char *s);
    extern int yylineno;
%}

%union {
   int i;      // za vracanje numerickih vrednosti
   char *s;    // za vracanje stringova
}

%token _TYPE
%token _IF
%token _ELSE
%token _RETURN
%token _DO
%token _WHILE
%token _ID
%token _INT_NUMBER
%token _LPAREN
%token _RPAREN
%token _LBRACKET
%token _RBRACKET
%token _ASSIGN
%token _SEMICOLON
%token _AROP
%token _RELOP
%token _INC
%token _COMMA
%token _AND
%token _OR


%%

program
   :  function_list
	|  variable_list function_list
   ;

function_list
   :  function
   |  function_list function
   ;

function
   :  type _ID _LPAREN _RPAREN body
   ;

type
   :  _TYPE
   ;

body
   :  _LBRACKET variable_list statement_list _RBRACKET
   |  _LBRACKET statement_list _RBRACKET
   ;

variable_list
   :  variable _SEMICOLON
   |  variable_list variable _SEMICOLON
   ;

variable
   :  type _ID
   |  variable _COMMA _ID
   ;

statement_list
   :  /* empty */
   |  statement_list statement
   ;

statement
   :  compound_statement
   |  assignment_statement
   |  if_statement
   |  return_statement
   |  do_statement
   |  inc_statement
   ;

do_statement
   :  _DO statement _WHILE _LPAREN rel_exp _RPAREN _SEMICOLON
   ;

inc_statement
   :  _ID _INC _SEMICOLON
   ;


compound_statement
   :  _LBRACKET statement_list _RBRACKET
   ;

assignment_statement
   :  _ID _ASSIGN num_exp _SEMICOLON
   ;

num_exp
   :  exp
   |  num_exp _AROP exp
   ;

exp
   :  constant
   |  _ID
   |  function_call
   |  _LPAREN num_exp _RPAREN
   |  _ID _INC
   ;

constant
   :  _INT_NUMBER
   ;

function_call
   :  _ID _LPAREN _RPAREN
   ;

if_statement
   :  if_part
   |  if_part _ELSE statement
   ;

if_part
   :  _IF _LPAREN log_exp _RPAREN statement
   ;

log_exp
   :	rel_exp
   |  log_exp _AND rel_exp
   |  log_exp _OR  rel_exp
   ;

rel_exp
   :  num_exp _RELOP num_exp
   ;

return_statement
   :  _RETURN num_exp _SEMICOLON
   ;




%%

int yyerror(char *s) {
   fprintf(stderr, "\nline %d: ERROR: %s", yylineno, s);
   return 0;
}

int main() {
   printf("\nSTART\n");
   yyparse();
   printf("\nSTOP\n");
   return 0;
}
